<?php

declare(strict_types=1);

namespace app\models;

use yii;

use app\models\traits\SearchByEdge;
use app\models\Account;

/**
 * Импорт поставок
 *
 * Хранит в себе связи с поставками которые были загружены вместе
 */
class Import extends Document
{
    use SearchByEdge;

    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return 'import';
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'name',
                'file'
            ]
        );
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'name' => 'Название',
                'file' => 'Файл'
            ]
        );
    }

    /**
     * Правила
     */
    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    [
                        'file',
                        'name'
                    ],
                    'string'
                ]
            ]
        );
    }

    /**
     * Поиск по складу
     *
     * @param Warehouse $warehouse Инстанция склада
     * @param int $limit Ограничение по максимальному количеству
     *
     * @return array Инстанции испортов
     */
    public static function searchByWarehouse(Warehouse $warehouse, int $limit = 10): array
    {
        return self::searchByEdge(
            from: 'warehouse',
            to: 'import',
            edge: 'warehouse_edge_import',
            direction: 'INBOUND',
            subquery_where: [
                ['warehouse_edge_import._from' => $warehouse->readId()],
                ['warehouse_edge_import.type' => 'loaded']
            ],
            where: 'warehouse_edge_import[0] != null',
            limit: $limit
        );
    }
}
